import os
import re
import argparse

class BinaryDataGenerater:
    def __init__(self):
        self.update_mode = 'new'
        self.filter = None
        self.check_out = False
        self.debug = False
        # valid scripts directory
        self.scripts_dir = ['st', 'ut']
        self.scripts = []
        self.work_path = []

        absolute_path = os.path.dirname(os.path.abspath(__file__))
        home_path, _ = absolute_path.split(os.path.basename(absolute_path))
        for dir in self.scripts_dir:
            self.work_path.append(os.path.join(home_path, dir))

    def print_params(self):
        print(f'-----------------------------------------------------------------------------------\n' \
              f'run params:\n'                                                                         \
              f'update_mode={self.update_mode}\n'                                                      \
              f'filter={self.filter}\n'                                                                \
              f'check_out={self.check_out}\n'                                                          \
              f'scripts_dir={self.scripts_dir}\n'                                                      \
              f'work_path={self.work_path}\n'                                                          \
              f'-----------------------------------------------------------------------------------')

    def print_scripts(self):
        print(f'-----------------------------------------------------------------------------------\n' \
              f'run scripts:')
        for script in self.scripts:
            print(script)
        print(f'-----------------------------------------------------------------------------------')

    def get_valid_scripts(self):
        for path in self.work_path:
            for root, dirs, files in os.walk(path):
                for file in files:
                    full_path = os.path.join(root, file)
                    if not full_path.endswith('.py') or full_path == '__init__.py':
                        continue
                    if self.filter is not None and self.filter not in full_path:
                        continue
                    self.scripts.append(full_path)
        self.scripts.sort()

    def check_out_binary_data(self):

        def delete_binary_data_without_script(wait_delete_list, pattern):
            for file in wait_delete_list:
                split = re.split(pattern, file)
                script = split[0] + '.py'
                if not os.path.exists(script):
                    print(f'delete {file}')
                    os.remove(file)

        if not self.check_out:
            return

        print('Start to checkout benchmark binary data...')
        binary_data_list = []
        for path in self.work_path:
            for root, dirs, files in os.walk(path):
                for file in files:
                    if not file.endswith('.npy'):
                        continue
                    full_path = os.path.join(root, file)
                    binary_data_list.append(full_path)

        binary_data_list.sort()
        input_data_list = [item for item in binary_data_list if re.search(r'[\s\S]*?_input(\d+).npy', item)]
        output_data_list = [item for item in binary_data_list if re.search(r'[\s\S]*?_output(\d+).npy', item)]
        delete_binary_data_without_script(input_data_list, r'_input(\d+).npy')
        delete_binary_data_without_script(output_data_list, r'_output(\d+).npy')

    def filter_scripts(self):

        def is_script_has_data(script):
            data_name, _ = os.path.splitext(script)
            input_data = data_name + '_input0.npy'
            output_data = data_name + '_output0.npy'
            if os.path.exists(input_data) or os.path.exists(output_data):
                return True

            return False

        if self.update_mode == 'all' and self.filter is None:
            return
        
        if self.filter:
            pattern = r'' + re.escape(self.filter)
            self.scripts = [script for script in self.scripts if re.search(pattern, script)]

        if self.update_mode == 'new':
            self.scripts = [script for script in self.scripts if not is_script_has_data(script)]

    def run_scripts(self):
        self.print_scripts()

        for script in self.scripts:
            print(f'python {script}')
            os.system(f'python {script}')

    def generate_data(self):
        print('Start to generate benchmark binary data...')
        self.print_params()
        # get all valid scripts.
        self.get_valid_scripts()
        # delete binary data without script if checkout is enable.
        self.check_out_binary_data()
        # get run scripts with update mode and filter
        self.filter_scripts()
        # run all scripts
        self.run_scripts()


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument("-m", "--update_mode", type=str, choices=['all', 'new'],
                        help="if 'all', force update all binary data. if 'new', only update incremental data.")
    parser.add_argument("-f", "--filter", type=str,
                        help="filter the binary benchmark scripts. like use 'ops' to filter 'ops_xxx.py'")
    parser.add_argument("-c", "--check_out", action="store_true",
                        help='Check out binary data, delete binary data without script.')
    parser.add_argument("-d", "--debug", action="store_true", help='Enable debug mode.')

    generater = BinaryDataGenerater()
    args = parser.parse_args()
    if args.update_mode:
        generater.update_mode = args.update_mode
    if args.filter:
        generater.filter = args.filter
    generater.debug = args.debug
    generater.check_out = args.check_out
    generater.generate_data()
